from __future__ import generators
import plus
import Arenas
import Hazards
import math
import random
from random import randint
#import Gooey

class BoxArena(Arenas.SuperArena):
    "Keznorg IGC. The Keznorg Intergalactic Center hosts some of the fiercest competition you will find. An electric fence guards the perimiter of the platform arena. Many amateurs are forbidden to compete here because the spectator fees are high and the crowd wants to see only the best."
    name = "Keznorg IGC"
    preview = "keznorg/Th3L3.bmp"
    game_types = ['DEATHMATCH', 'BATTLE ROYAL', 'TEAM MATCH']
    extent = (-12, 12, 12, -12)

    def __init__(self):
        Arenas.SuperArena.__init__(self, "Arenas/keznorg/keznorg.gmf")
        plus.setBackColor(0,0,0)
        degrad = 0.01745
        self.sensors = {}
        self.zaptimer = 1
        self.bgtimer = 0
        self.color = 0
        self.zapsound = plus.createSound("Sounds/zap_loop.wav", False, (0,0,0))
        self.AddStaticCamera("Static 1", (-20, 8, 15), (30*degrad, 128*degrad), degrad*55)
        self.AddStaticCamera("Static 2", (0, 30, 0), (90*degrad, 90*degrad), degrad*55)
        self.AddStaticCamera("Static 3", (0, -20, 0), (-90*degrad, 0), degrad*75)
        self.AddStaticCamera("Static 4", (0, 7.8, -18), (40*degrad, 0), degrad*220)

        self.AddWatchCamera("Watch Cam 1", (20, 15, -15), (20, 40, 50*degrad, 40*degrad))
        self.AddWatchCamera("Watch Cam 2", (-20, 8, 15), (12, 20, 40*degrad, 30*degrad))
        self.AddWatchCamera("Watch Cam 3", (0, 0, 0), (2, 14, 315*degrad, 10*degrad))
        self.AddWatchCamera("Watch Cam 4", (0, -20, 0), (20, 35, 50*degrad, 40*degrad))
        self.players = ()
        
        # self.plah = Gooey.Plain("blokus", 40, 90, 250, 120)
        # self.tbox0 = self.plah.addText("uno", 0, 0, 250, 15)
        # self.tbox0.setText("")

    def Activate(self, on):
        if on: self.players = plus.getPlayers()
        
        Arenas.SuperArena.Activate(self, on)
        
    def HazardsOn(self, on):
        #walls
        self.AddCollisionLine((-12, 12), (12, 12))
        self.AddCollisionLine((12, 12), (12, -12))
        self.AddCollisionLine((12, -12), (-12, -12))
        self.AddCollisionLine((-12, -12), (-12, 12))
        plus.Arena.__setattr__tickInterval__(Arenas.currentArena, 0.05)
        #make smart zone rotate so bots aren't constantly zapped
        self.zapzone = self.GetHinge("Hinge01")
        self.zapzone.SetAutoLocks(False, False)
        self.zapzone.Lock(False)
        self.zapzone.SetPowerSettings(300,150)
        self.zapzone.SetDirection(100)
            
        return Arenas.SuperArena.HazardsOn(self, on)
        
    def __del__(self):
        plus.stopAllSounds()
        Arenas.SuperArena.__del__(self)
        
    def Tick(self):
        #self.tbox0.setText("Bots in zone: " + str(self.sensors))
        # Electric fence
        if self.bHazardsOn:
            for bot, in_range in self.sensors.iteritems():
                if in_range:
                    plus.damage(bot, 0, self.zaptimer, plus.getLocation(bot))
                    plus.zap(bot, self.zaptimer, 0.25)
                    plus.loopSound(self.zapsound)
                    
            if self.NumBotsInRange()==0:
                plus.stopSound(self.zapsound)
                self.zaptimer = 1
            else:
                if self.zaptimer < 50:
                    self.zaptimer += 1
        if not plus.isMatchOver():
            #make electricity light up arena
            self.color = randint(0, 6)
            self.bgtimer += 1
            if self.bgtimer == 0 and self.color == 0:
                plus.setBackColor(0,0,0)
            if self.bgtimer == 0 and self.color == 1:
                plus.setBackColor(1,0,0)
            if self.bgtimer == 0 and self.color == 2:
                plus.setBackColor(1,1,0)
            if self.bgtimer == 0 and self.color == 3:
                plus.setBackColor(0,1,0)
            if self.bgtimer == 0 and self.color == 4:
                plus.setBackColor(0,1,1)
            if self.bgtimer == 0 and self.color == 5:
                plus.setBackColor(0,0,1)
            if self.bgtimer == 0 and self.color == 6:
                plus.setBackColor(1,0,1)
            if self.bgtimer == 1:
                plus.setBackColor(1,1,1)
                self.bgtimer = -1
        else:
            plus.setBackColor(0,0,0)
                
        #check to see if anyone has been "eliminated" by falling off the tabletop
        #why is "eliminated" in quotes?
        for each in self.players:
            if plus.getLocation(each)[1] < -5:
                plus.eliminatePlayer(each)
                    
        return Arenas.SuperArena.Tick(self)
        
    def NumBotsInRange(self):
        numBots = 0
        for bot, in_range in self.sensors.iteritems():
            if in_range: numBots += 1
                
        return numBots
        
    def ZoneEvent(self, direction, id, robot, chassis):
        r = robot - 1
        if r not in self.sensors: self.sensors[r] = 0
        
        if direction==1:
            self.sensors[r] += 1
        elif direction==-1:
            self.sensors[r] -= 1
            
        return True
        
Arenas.register(BoxArena)